@@ -20,6 +20,8 @@
 #    along with ReSiStance.  If not, see <http://www.gnu.org/licenses/>.
 #########################################################################
 
+from asyncworker import AsyncWorker, AsyncItem, PriorityQueue
+from Queue import Queue
 import base64
 import constants
 import cPickle
@@ -97,6 +99,10 @@ class ReSiStanceFeedDict(feedparser.FeedParserDict):
 
 class FeedManager(gobject.GObject):
 
+    LOW_PRIORITY = 10
+    DEFAULT_PRIORITY = 0
+    HIGH_PRIORITY = -10
+
     def __init__(self, settings, conn_manager):
         super(FeedManager, self).__init__()
 
@@ -106,6 +112,29 @@ class FeedManager(gobject.GObject):
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
         self._google_reader_auth = None
         self._google_reader_auth_token = None
+        self._async_worker = None
+
+    def stop(self):
+        # Cancel all pending operations
+        self._async_worker.stop()
+
+    def halt(self):
+        # Wait till all the workers are done with their tasks
+        self._async_worker.halt()
+
+    def requireWorker(f):
+        '''Function decorator to ensure that a valid async worker
+        exists and is properly initialized in order to run the called
+        method'''
+        def wrapper(self, *args):
+            # Check if we need to create the AsyncWorker. Use 2 worker threads
+            if not self._async_worker:
+                self._async_worker = AsyncWorker(2)
+
+            # Call the async function
+            f(self, *args)
+
+        return wrapper
 
     def _on_connection_changed(self, conn_manager):
         pass
@@ -113,33 +142,104 @@ class FeedManager(gobject.GObject):
     def get_feed_list(self):
         return self.feed_data_list
 
-    def _safe_callback(self, callback, *args):
-        ''' Calls a callback if exists using gtk guards '''
-        if callback:
-            gtk.gdk.threads_enter()
-            callback(*args)
-            gtk.gdk.threads_leave()
-
+    @requireWorker
     def add_feed(self, url, sync, save, callback, data=None):
+        # Create a new worker to update from network
+        add_item = AsyncItem(self._add_feed_in_thread, (url,sync,save,),
+                             callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, add_item))
 
-        # Create a new thread to update from network
-        adding_thread = Thread(target=self._add_feed_in_thread,
-                               args=(url, sync, save, callback, data))
-        adding_thread.start()
+    def _add_feed_in_thread(self, url, sync, save):
 
+        parsed_url = urlparse.urlsplit(url)
+        if parsed_url.scheme == '':
+            url = 'http://' + url
+
+        if not url.endswith('xml') and not url.endswith('opml'):
+            url = getRSSLink(url)
+
+        # Return if we cannot get the feed URL
+        if url == '':
+            raise IOError('Invalid URL')
+
+        new_feed_data = ReSiStanceFeedDict(feedparser.parse(url), sync)
+
+        # 200 == OK, and 3xx are redirections.
+        # On the other hand bozo==1 if there was some problem parsing the feed
+        if (new_feed_data.status!=200 and new_feed_data.status/100 != 3) or \
+                new_feed_data.bozo:
+            del new_feed_data
+            raise IOError('Resource is not a feed')
+
+        self.feed_data_list.append(new_feed_data)
+
+        if 'link' in new_feed_data.feed:
+            pixbuf = self._get_favicon_sync(new_feed_data.feed.link)
+        else:
+            pixbuf = self._get_favicon_sync(new_feed_data.href)
+
+        # Save to disk
+        if save:
+            try:
+                self.save(None)
+            except IOError:
+                pass
+
+        return pixbuf, new_feed_data
+
+    @requireWorker
     def update_feed(self, feed, callback, data=None):
+        # Create a new worker to update from network
+        update_item = AsyncItem(self._update_feeds_in_thread, ([feed],),
+                                callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, update_item))
 
-        # Create a new thread to update from network
-        updating_thread = Thread(target=self._update_feeds_in_thread,
-                                 args=([feed], callback, data))
-        updating_thread.start()
-
+    @requireWorker
     def update_all(self, callback, data):
+        # Create a new worker to update from network
+        update_item = AsyncItem(self._update_feeds_in_thread, (self.feed_data_list,),
+                                callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, update_item))
 
-        # Create a new thread to update from network
-        updating_thread = Thread(target=self._update_feeds_in_thread,
-                                 args=(self.feed_data_list, callback, data))
-        updating_thread.start()
+    def _update_feeds_in_thread(self, feed_data_list):
+
+        for feed_data in feed_data_list:
+            updated_feed_data = ReSiStanceFeedDict(feedparser.parse(feed_data.href))
+
+            # In case of network failure
+            if updated_feed_data == None or updated_feed_data.entries == None or \
+                    len(updated_feed_data.entries) == 0:
+                continue
+
+            updated_feed_date = updated_feed_data.feed.get('updated_parsed') or \
+                updated_feed_data.entries[0].get('updated_parsed')
+            feed_date = feed_data.feed.get('updated_parsed') or \
+                feed_data.entries[0].get('updated_parsed')
+
+            if updated_feed_date > feed_date:
+                old_entry_ids = [entry.id for entry in feed_data.entries]
+                updated_entry_ids = [entry.id for entry in updated_feed_data.entries]
+                new_entries = [entry for entry in updated_feed_data.entries if entry.id not in old_entry_ids]
+                del old_entry_ids, updated_entry_ids
+
+                # Autodownload enclosures
+                if self.settings.auto_download:
+                    to_download_urls = [entry.enclosures[0].href for entry in new_entries if 'enclosures' in entry]
+                    to_download_paths = [self.settings.auto_download_folder+os.path.basename(urllib.url2pathname(url)) for url in to_download_urls]
+                    self._download_items_in_thread(to_download_urls, to_download_paths)
+
+                # In order to keep read/unread status, just add
+                # new_entries to the old ones while updating the rest
+                # of the information
+                old_entries = feed_data.entries
+                feed_data.update(updated_feed_data)
+                feed_data.entries = old_entries + new_entries
+
+        # Save to disk
+        try:
+            self.save(None)
+        except IOError:
+            pass
 
     def remove_feed(self, feed):
         self.feed_data_list.remove(feed)
@@ -150,17 +250,14 @@ class FeedManager(gobject.GObject):
         except IOError:
             pass
 
+    @requireWorker
     def export_opml(self, file_path, callback, data=None):
+        # Create a new worker to save the  file
+        export_item = AsyncItem(self._export_opml_in_thread, (file_path,),
+                                callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, export_item))
 
-        # Create a new thread to update from network
-        exporting_thread = Thread(target=self._export_opml_in_thread,
-                               args=(file_path, callback, data))
-        try:
-            exporting_thread.start()
-        except IOError:
-            callback(False)
-
-    def _export_opml_in_thread(self, file_path, callback, user_data):
+    def _export_opml_in_thread(self, file_path):
         # Create the minidom document
         opml_doc = Document()
 
@@ -180,67 +277,52 @@ class FeedManager(gobject.GObject):
         root.appendChild(body)
         opml_doc.appendChild(root)
 
-        try:
-            retval = True
-            file_opml = open(file_path,"w")
-            opml_doc.writexml(file_opml, "    ", "", "\n", "UTF-8")
-        except IOError :
-            print 'Error exporting OPML, could not write to ' + file_path
-            retval = False
-
-        self._safe_callback(callback, retval)
+        file_opml = open(file_path,"w")
+        opml_doc.writexml(file_opml, "    ", "", "\n", "UTF-8")
 
+    @requireWorker
     def import_opml(self, file_path, callback, data=None):
+        # Create a new worker to read the file
+        import_item = AsyncItem(self._import_opml_in_thread, (file_path,),
+                                callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, import_item))
 
-        # Create a new thread to update from network
-        importing_thread = Thread(target=self._import_opml_in_thread,
-                               args=(file_path, callback, data))
-        try:
-            importing_thread.start()
-        except IOError:
-            callback(None, None)
-
-    def _import_opml_in_thread(self, file_path, callback, user_data):
+    def _import_opml_in_thread(self, file_path):
 
         feed_url_list = [];
 
         if (not (os.path.exists(file_path) and os.path.isfile(file_path)) or
             (os.path.splitext(file_path)[1] != '.opml')):
-            self._safe_callback(callback, feed_url_list, user_data)
-            return
+            return feed_url_list
 
-        try:
-            doc=open(file_path,'r')
-            opml_doc = minidom.parse(doc)
-            opml_doc.getElementsByTagName('outline')
+        doc=open(file_path,'r')
+        opml_doc = minidom.parse(doc)
+        opml_doc.getElementsByTagName('outline')
 
-            feed_url_list = [node.attributes['xmlUrl'].value \
+        feed_url_list = [node.attributes['xmlUrl'].value \
                              for node in opml_doc.getElementsByTagName('outline') \
                              if node.getAttribute('xmlUrl') != '']
-        except IOError :
-            self._safe_callback(callback, feed_url_list, user_data)
-            return
-
         doc.close()
-        self._safe_callback(callback, feed_url_list, user_data)
 
-    def find_feed(self, key_words, dialog, callback):
-        # Create a new thread to search
-        find_thread = Thread(target=self._find_feed_in_thread,
-                               args=(key_words, dialog, callback))
+        return feed_url_list
 
-        return find_thread.start()
+    @requireWorker
+    def find_feed(self, keywords, callback, data):
+        # Create a new worker to search
+        find_item = AsyncItem(self._find_feed_in_thread, (keywords,),
+                              callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, find_item))
 
-    def _find_feed_in_thread(self, key_words, dialog, callback):
+    def _find_feed_in_thread(self, keywords):
         try:
             server = xmlrpclib.Server('http://www.syndic8.com/xmlrpc.php')
-            feedids = server.syndic8.FindFeeds(key_words,'last_pubdate',25,0)
+            feedids = server.syndic8.FindFeeds(keywords,'last_pubdate',25,0)
             infolist = server.syndic8.GetFeedInfo(feedids, ['imageurl','sitename','dataurl'])
         except:
             infolist = None
             print 'Error while accessing syndic8.com'
 
-        self._safe_callback(callback, key_words, dialog, infolist)
+        return infolist
 
     def _authenticate_google_reader(self):
         ''' Returns a pair of authentication headers and the authentication token '''
@@ -274,21 +356,20 @@ class FeedManager(gobject.GObject):
 
         return auth_headers, self._google_reader_auth_token
 
+    @requireWorker
     def mark_as_read_synchronize(self, feed_data_base, item_ref, read_item=True, callback=None, data=None):
         # Create a new thread to synchronize
-        synchronize_read_thread = Thread(target=self._mark_as_read_synchronize_in_thread,
-                                         args=(feed_data_base, item_ref, read_item, callback, data))
+        mark_read_item = AsyncItem(self._mark_as_read_synchronize_in_thread, (feed_data_base, item_ref, read_item),
+                                   callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, mark_read_item))
 
-        return synchronize_read_thread.start()
-
-    def _mark_as_read_synchronize_in_thread(self, feed_data_base, item_ref, read_item, callback, user_data):
+    def _mark_as_read_synchronize_in_thread(self, feed_data_base, item_ref, read_item):
         synced = False
         read_mark = "a" if read_item else "r"
 
         auth_headers, auth_token = self._authenticate_google_reader()
         if not auth_token:
-            self._safe_callback(callback, synced, user_data)
-            return
+            return False
 
         try:
             # Get feed
@@ -296,8 +377,7 @@ class FeedManager(gobject.GObject):
             req = urllib2.Request(feed_request, headers = auth_headers)
             feed_response = urllib2.urlopen(req)
         except:
-            self._safe_callback(callback, synced, user_data)
-            return
+            return False
 
         response_data=feed_response.read()
 
@@ -305,8 +385,7 @@ class FeedManager(gobject.GObject):
             try:
                 doc = minidom.parseString(response_data)
             except:
-                self._safe_callback(callback, synced, user_data)
-                return
+                return False
             #retrieve following 20 items
             nodes = doc.getElementsByTagName('gr:continuation')
             nodesEntries = doc.getElementsByTagName('entry')
@@ -329,32 +408,27 @@ class FeedManager(gobject.GObject):
                     r = urllib2.urlopen(req)
                     response_data = feed_response.read()
             except:
-                self._safe_callback(callback, synced, user_data)
-                return
+                return False
 
         # Call user callback
-        self._safe_callback(callback, synced, user_data)
+        return synced
 
-    def download_item(self, url, path_file, callback, data=None):
-        # Create a new thread to download from network
-        downloading_thread = Thread(target=self._download_items_in_thread,
-                               args=([url], [path_file], callback, data))
-        try:
-            downloading_thread.start()
-        except IOError:
-            callback(False)
+    @requireWorker
+    def download_item(self, url, file_path, callback, data=None):
+        # Create a new worker to download from network
+        download_item = AsyncItem(self._download_items_in_thread, ([url], [file_path]),
+                                 callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, download_item))
 
-    def download_all_items(self, urls, paths_files, callback, data=None):
-        # Create a new thread to download items from network
-        downloading_all_thread = Thread(target=self._download_items_in_thread,
-                               args=(urls, paths_files, callback, data))
-        try:
-            downloading_all_thread.start()
-        except IOError:
-            callback(False)
+    @requireWorker
+    def download_all_items(self, urls, file_paths, callback, data=None):
+        # Create a new worker to download from network
+        download_item = AsyncItem(self._download_items_in_thread, (urls, file_paths),
+                                 callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, download_item))
 
-    def _download_items_in_thread(self, urls, paths_files, callback, data):
-        path_file = iter(paths_files)
+    def _download_items_in_thread(self, urls, file_paths):
+        path_file = iter(file_paths)
         retvalues = []
         for url in urls:
             try:
@@ -366,21 +440,20 @@ class FeedManager(gobject.GObject):
                 retvalues.append(False)
                 break
 
-        self._safe_callback(callback, False not in retvalues)
+        return False not in retvalues
 
+    @requireWorker
     def sync_with_google_reader(self, callback, data=None):
-        # Create a new thread to search
-        sync_with_google_reader_thread = Thread(target=self._sync_with_google_reader_in_thread,
-                                                args=(callback, data))
+        # Create a new worker to sync
+        sync_item = AsyncItem(self._sync_with_google_reader_in_thread, (),
+                              callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, sync_item))
 
-        return sync_with_google_reader_thread.start()
-
-    def _sync_with_google_reader_in_thread(self, callback, user_data):
+    def _sync_with_google_reader_in_thread(self):
 
         auth_headers, auth_token = self._authenticate_google_reader()
         if not auth_token:
-            self._safe_callback(callback, None, user_data)
-            return
+            return None
 
         urls = []
         try:
@@ -410,8 +483,9 @@ class FeedManager(gobject.GObject):
         except:
             pass
 
-        self._safe_callback(callback, urls, user_data)
+        return urls
 
+    @requireWorker
     def save(self, callback=None, data=None):
         # TODO: migrate to "with" statement when available
         try:
@@ -420,20 +494,27 @@ class FeedManager(gobject.GObject):
             print 'Cannot write to', constants.RSS_DB_FILE
             raise
         else:
-            # Create a new thread to store in disk
-            saving_thread = Thread(target=self._save_in_thread,
-                                   args=(db_file, callback, data))
-            saving_thread.start()
+            # Create a new worker to store in disk
+            save_item = AsyncItem(self._save_in_thread, (db_file,),
+                                  callback, (data,))
+            self._async_worker.add_task((self.LOW_PRIORITY, save_item))
 
-    def _save_in_thread(self, db_file, callback, data):
+    def save_sync(self):
+        # TODO: migrate to "with" statement when available
+        try:
+            db_file = open(constants.RSS_DB_FILE, 'w')
+            cPickle.dump(self.feed_data_list, db_file)
+        except:
+            return
+
+    def _save_in_thread(self, db_file):
         try:
             cPickle.dump(self.feed_data_list, db_file)
         except cPickle.UnpickleableError:
             print 'Cannot serialize to', constants.RSS_DB_FILE
             raise
 
-        self._safe_callback(callback, data)
-
+    @requireWorker
     def load(self, callback, data=None):
         # TODO: migrate to "with" statement when available
         try:
@@ -442,26 +523,19 @@ class FeedManager(gobject.GObject):
             print 'Cannot open', constants.RSS_DB_FILE
             raise
         else:
-            # Create a new thread to load from disk
-            loading_thread = Thread(target=self._load_in_thread,
-                                   args=(db_file, callback, data))
-            loading_thread.start()
+            load_item = AsyncItem(self._load_in_thread, (db_file,),
+                                  callback, (data,))
+            self._async_worker.add_task((self.DEFAULT_PRIORITY, load_item))
 
-    def _load_in_thread(self, db_file, callback, data):
+    def _load_in_thread(self, db_file):
         feed_data_list = cPickle.load(db_file)
         self.feed_data_list = [ReSiStanceFeedDict(feed_data) for feed_data in feed_data_list]
 
-        self._safe_callback(callback, data)
-
+    @requireWorker
     def get_favicon(self, url, callback, data=None):
-        get_favicon_thread = Thread(target=self._get_favicon_in_thread,
-                                    args=(url, callback, data))
-        get_favicon_thread.start()
-
-    def _get_favicon_in_thread(self, url, callback, data):
-        favicon = self._get_favicon_sync(url)
-
-        self._safe_callback(callback, favicon, data)
+        get_favicon_item = AsyncItem(self._get_favicon_sync,(url,),
+                                     callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, get_favicon_item))
 
     def _get_favicon_sync(self, url):
         # Check that user dir exists
@@ -510,21 +584,20 @@ class FeedManager(gobject.GObject):
 
         return pixbuf
 
+    @requireWorker
     def subscribe_feed_google(self, feed_data, feed_url, is_add_subcription, callback=None, user_data=None):
         # Create a new thread to synchronize(subscribe and unsubscribe)
-        subscribe_feed_google_thread = Thread(target=self._subscribe_feed_google_in_thread,
-                                              args=(feed_data, feed_url, is_add_subcription, callback, user_data))
+        subscribe_item = AsyncItem(self._subscribe_feed_google_in_thread, (feed_data,feed_url,is_add_subcription),
+                                   callback, (user_data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, subscribe_item))
 
-        return subscribe_feed_google_thread.start()
-
-    def _subscribe_feed_google_in_thread(self, feed_data, feed_url, is_add_subcription, callback, data):
+    def _subscribe_feed_google_in_thread(self, feed_data, feed_url, is_add_subcription):
         synced = False
         feed_data.sync = is_add_subcription
 
         auth_headers, auth_token = self._authenticate_google_reader()
         if not auth_token:
-            self._safe_callback(callback, synced, user_data)
-            return
+            return synced
 
         action = 'subscribe' if is_add_subcription else 'unsubscribe'
         try:
@@ -537,21 +610,20 @@ class FeedManager(gobject.GObject):
         else:
             synced = True
 
-        self._safe_callback(callback, synced, data)
+        return synced
 
+    @requireWorker
     def sync_google_reader_read_status(self, callback=None, user_data=None):
         ''' Synchronize the read/unread status of all (mandatory by
         Google Reader API) subscribed feeds with Google Reader '''
-        sync_read_status_thread = Thread(target=self._sync_google_reader_read_status_in_thread,
-                                         args=(callback, user_data))
+        sync_read_item = AsyncItem(self._sync_google_reader_read_status_in_thread, (),
+                                   callback, (user_data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, sync_read_item))
 
-        return sync_read_status_thread.start()
-
-    def _sync_google_reader_read_status_in_thread(self, callback, data):
+    def _sync_google_reader_read_status_in_thread(self):
         auth_headers, auth_token = self._authenticate_google_reader()
         if not auth_token:
-            self._safe_callback(callback, False, data)
-            return
+            return False
 
         try:
             # Get items from feed excluding those with read status. It
@@ -562,8 +634,7 @@ class FeedManager(gobject.GObject):
             unread_response = urllib2.urlopen(unread_request)
             unread_data = unread_response.read()
         except:
-            self._safe_callback(callback, False, data)
-            return
+            return False
 
         unread_dict = {}
         while True:
@@ -588,8 +659,7 @@ class FeedManager(gobject.GObject):
                 unread_response = urllib2.urlopen(unread_request)
                 unread_data = unread_response.read()
             except:
-                self._safe_callback(callback, False, data)
-                return
+                return False
 
         for key in unread_dict.keys():
             # This could happen if there are unread items in feeds
@@ -608,7 +678,7 @@ class FeedManager(gobject.GObject):
                 if entry.link in unread_dict[key] or entry.link[:entry.link.find('?')] in unread_dict[key]:
                     # If read in ReSiStance update Google Reader (most likely read while offline)
                     if entry.read:
-                        self._mark_as_read_synchronize_in_thread(feed_data.feed.link, entry.link, True, None, None)
+                        self._mark_as_read_synchronize_in_thread(feed_data.feed.link, entry.link, True)
                 else:
                     # If read in Google Reader update ReSiStance (most likely read in Web)
                     if not entry.read:
@@ -622,88 +692,4 @@ class FeedManager(gobject.GObject):
                 if not entry.read:
                     entry.read = True
 
-        self._safe_callback(callback, True, data)
-
-    def _add_feed_in_thread(self, url, sync, save, callback, user_data):
-
-        parsed_url = urlparse.urlsplit(url)
-        if parsed_url.scheme == '':
-            url = 'http://' + url
-
-        if not url.endswith('xml') and not url.endswith('opml'):
-            url = getRSSLink(url)
-
-        # Return if we cannot get the feed URL
-        if url == '':
-            self._safe_callback(callback, None, None, user_data)
-            return
-
-        new_feed_data = ReSiStanceFeedDict(feedparser.parse(url), sync)
-
-        # 200 == OK, and 3xx are redirections.
-        # On the other hand bozo==1 if there was some problem parsing the feed
-        if (new_feed_data.status!=200 and new_feed_data.status/100 != 3) or \
-                new_feed_data.bozo:
-            self._safe_callback(callback, None, None, user_data)
-            return
-
-        self.feed_data_list.append(new_feed_data)
-
-        if 'link' in new_feed_data.feed:
-            pixbuf = self._get_favicon_sync(new_feed_data.feed.link)
-        else:
-            pixbuf = self._get_favicon_sync(new_feed_data.href)
-
-        # Call user callback
-        self._safe_callback(callback, pixbuf, new_feed_data, user_data)
-
-        # Save to disk
-        if save:
-            try:
-                self.save(None)
-            except IOError:
-                pass
-
-    def _update_feeds_in_thread(self, feed_data_list, callback, user_data):
-
-        for feed_data in feed_data_list:
-            updated_feed_data = ReSiStanceFeedDict(feedparser.parse(feed_data.href))
-
-            # In case of network failure
-            if updated_feed_data == None or updated_feed_data.entries == None or \
-                    len(updated_feed_data.entries) == 0:
-                continue
-
-            updated_feed_date = updated_feed_data.feed.get('updated_parsed') or \
-                updated_feed_data.entries[0].get('updated_parsed')
-            feed_date = feed_data.feed.get('updated_parsed') or \
-                feed_data.entries[0].get('updated_parsed')
-
-            if updated_feed_date > feed_date:
-                old_entry_ids = [entry.id for entry in feed_data.entries]
-                updated_entry_ids = [entry.id for entry in updated_feed_data.entries]
-                new_entries = [entry for entry in updated_feed_data.entries if entry.id not in old_entry_ids]
-                del old_entry_ids, updated_entry_ids
-
-                # Autodownload enclosures
-                if self.settings.auto_download:
-                    to_download_urls = [entry.enclosures[0].href for entry in new_entries if 'enclosures' in entry]
-                    to_download_paths = [self.settings.auto_download_folder+os.path.basename(urllib.url2pathname(url)) for url in to_download_urls]
-                    self._download_items_in_thread(to_download_urls, to_download_paths, None, None)
-
-                # In order to keep read/unread status, just add
-                # new_entries to the old ones while updating the rest
-                # of the information
-                old_entries = feed_data.entries
-                feed_data.update(updated_feed_data)
-                feed_data.entries = old_entries + new_entries
-
-        # Call user callback. Call it just once, if callers want a different behaviour
-        # they can always call update_feed() for each one
-        self._safe_callback(callback, user_data)
-
-        # Save to disk
-        try:
-            self.save(None)
-        except IOError:
-            pass
+        return True
