@@ -483,16 +483,25 @@ class FeedsWindow(hildon.StackableWindow):
         else:
             self.align.set_padding(4, 0 , 4, 4)
 
-    def _on_exit(self, window, event):
-        # Save the feeds status
-        self.hide_on_delete()
-        self.manager.save(self._on_feed_data_saved)
+    def _on_exit_idle(self, data):
+        # Wait until all tasks are canceled
+        self.manager.stop()
+        self.manager.halt()
+
         self.settings.save()
+
+        # Save items to disk without using workers
+        self.manager.save_sync()
+        gobject.idle_add(gtk.main_quit, None)
         return True
 
-    def _on_feed_data_saved(self, user_data):
-        # Close application
-        gtk.main_quit()
+    def _on_exit(self, window, event):
+        # Hide window
+        self.hide()
+
+        # We do the final operations in an idle in order to let the window close
+        gobject.idle_add(self._on_exit_idle, None)
+        return True
 
     def _create_menu(self):
         menu = hildon.AppMenu()
@@ -501,21 +510,21 @@ class FeedsWindow(hildon.StackableWindow):
         self.ascending_filter_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.ascending_filter_button.set_mode(False)
         self.ascending_filter_button.set_label(_('A-Z'))
-        self.ascending_filter_button.connect('clicked', self._sort_ascending_cb)
+        self.ascending_filter_button.connect('toggled', self._sort_ascending_cb)
         menu.add_filter(self.ascending_filter_button)
         self.descending_filter_button = \
             hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                   group = self.ascending_filter_button)
         self.descending_filter_button.set_mode(False)
         self.descending_filter_button.set_label(_('Z-A'))
-        self.descending_filter_button.connect('clicked', self._sort_descending_cb)
+        self.descending_filter_button.connect('toggled', self._sort_descending_cb)
         menu.add_filter(self.descending_filter_button)
         self.visits_filter_button = \
             hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
                                   group = self.ascending_filter_button)
         self.visits_filter_button.set_mode(False)
         self.visits_filter_button.set_label(_('Favorites'))
-        self.visits_filter_button.connect('clicked', self._sort_visits_cb)
+        self.visits_filter_button.connect('toggled', self._sort_visits_cb)
         menu.add_filter(self.visits_filter_button)
 
         self._new_feed_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
@@ -668,7 +677,7 @@ class FeedsWindow(hildon.StackableWindow):
         else:
             return ''
 
-    def _all_feeds_updated_cb(self, data):
+    def _all_feeds_updated_cb(self, retval, data, error):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
 
         # TODO: Update counters
@@ -677,6 +686,16 @@ class FeedsWindow(hildon.StackableWindow):
             unread_count = len([entry for entry in row[self.FEEDS_MODEL_DATA_COLUMN].entries if entry.read == False])
             row[self.FEEDS_MODEL_READ_COLUMN] = self._get_visual_unread_text(unread_count)
 
+        # Do not update read/unread status if all the feeds failed to sync
+        synced_feeds = [feed for feed in self.manager.get_feed_list() if feed.sync]
+        if len(synced_feeds) == 0:
+            return
+
+        # Update read/unread status
+        hildon.hildon_banner_show_information(self, '', _('Synchronizing read/unread status with Google Reader'))
+        hildon.hildon_gtk_window_set_progress_indicator(self, True)
+        self.manager.sync_google_reader_read_status(self._sync_google_reader_read_status_cb)
+
     def _new_feed_response_cb(self, dialog, response):
 
         if response == gtk.RESPONSE_ACCEPT:
@@ -696,11 +715,7 @@ class FeedsWindow(hildon.StackableWindow):
 
             dialog.destroy()
 
-    def _feed_added_cb(self, pixbuf, new_feed_data, row_reference=None, stop_progress=True):
-
-        feed_data_list = self.manager.get_feed_list()
-        if len(feed_data_list) > 0:
-            self.export_opml_button.show()
+    def _feed_added_cb(self, pixbuf_and_data, row_reference=None, error=None, stop_progress=True):
 
         # Remove progress information
         if stop_progress:
@@ -713,7 +728,7 @@ class FeedsWindow(hildon.StackableWindow):
             feed_iter = store.append()
 
         # Check that the feed data was retrieved
-        if new_feed_data == None:
+        if error:
             message = _('Could not retrieve feed')
             dummy_data = store.get_value (feed_iter, self.FEEDS_MODEL_DATA_COLUMN)
             if dummy_data:
@@ -722,6 +737,11 @@ class FeedsWindow(hildon.StackableWindow):
             hildon.hildon_banner_show_information(self, '', message)
             return
 
+        pixbuf, new_feed_data = pixbuf_and_data
+        feed_data_list = self.manager.get_feed_list()
+        if len(feed_data_list) > 0:
+            self.export_opml_button.show()
+
         # Update model
         store.set(feed_iter,
                   self.FEEDS_MODEL_PIXBUF_COLUMN, pixbuf,
@@ -733,9 +753,11 @@ class FeedsWindow(hildon.StackableWindow):
                   self.FEEDS_MODEL_SYNC_COLUMN, new_feed_data.sync,
                   self.FEEDS_MODEL_ID_COLUMN, hash(new_feed_data.href))
 
-    def _multiple_add_cb(self, pixbuf, new_feed_data, (num_operations, callback, data, row_reference), static = {"count" : 0}):
+    def _multiple_add_cb(self, pixbuf_and_data, user_data, error, static = {"count" : 0}):
         ''' Simulating a static variable with a default argument hack '''
-        self._feed_added_cb(pixbuf, new_feed_data, row_reference, stop_progress=False)
+        num_operations, callback, data, row_reference = user_data
+
+        self._feed_added_cb(pixbuf_and_data, row_reference, error, stop_progress=False)
         static['count'] += 1
         # If all add operations have finished then call the all-done-callback
         if static['count'] == num_operations:
@@ -759,7 +781,7 @@ class FeedsWindow(hildon.StackableWindow):
             self.manager.add_feed(url, sync, save, self._multiple_add_cb,
                                   (len(urls), callback, data, row_reference))
 
-    def _sync_google_reader_read_status_cb(self, synced, user_data):
+    def _sync_google_reader_read_status_cb(self, synced, user_data, error):
         if not synced:
             hildon.hildon_banner_show_information(self, '', _('Error syncing read status with Google Reader'))
             return
@@ -787,7 +809,7 @@ class FeedsWindow(hildon.StackableWindow):
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
         self.manager.sync_google_reader_read_status(self._sync_google_reader_read_status_cb)
 
-    def _feeds_synchronized(self, urls, user_data):
+    def _feeds_synchronized(self, urls, user_data, error):
 
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
         if urls == None:
@@ -838,8 +860,8 @@ class FeedsWindow(hildon.StackableWindow):
             # Update rotation
             self.rotation_object.set_mode(self.settings.rotation_mode)
 
-    def _feed_exported_cb(self, exported):
-        if exported:
+    def _feed_exported_cb(self, retval, user_data, error):
+        if not error:
             message = _('Feeds exported')
         else:
             message = _('Error exporting feeds')
@@ -899,7 +921,7 @@ class FeedsWindow(hildon.StackableWindow):
         sync_google_button.show()
         dialog.show()
 
-    def _feed_data_loaded(self, user_data):
+    def _feed_data_loaded(self, retval, user_data, error):
         # TODO: migrate to "with" statement when available
         try:
             feed_data_list = self.manager.get_feed_list()
@@ -932,7 +954,7 @@ class FeedsWindow(hildon.StackableWindow):
         if len(feed_data_list) == 0:
             self._show_initial_dialog()
 
-    def _get_favicon_cb(self, pixbuf, row_reference):
+    def _get_favicon_cb(self, pixbuf, row_reference, error):
         if pixbuf == None:
             return
 
@@ -942,12 +964,12 @@ class FeedsWindow(hildon.StackableWindow):
 
     def _find_feed_response_cb(self, dialog, response):
         if response == gtk.RESPONSE_ACCEPT:
-            key_words = dialog.entry.get_text()
+            keywords = dialog.entry.get_text()
 
             # Quickly show feedback to user
             hildon.hildon_gtk_window_set_progress_indicator(dialog, True)
 
-            self.manager.find_feed(key_words, dialog, self._found_cb)
+            self.manager.find_feed(keywords, self._found_cb, dialog)
             dialog.set_response_sensitive(gtk.RESPONSE_ACCEPT, False)
 
             # Disable canceling
@@ -960,14 +982,14 @@ class FeedsWindow(hildon.StackableWindow):
         if len(urls):
             self._add_multiple_feeds(urls, callback=self._save_feeds_after_multiple_add)
 
-    def _found_cb(self, key_words, dialog, feeds_info):
+    def _found_cb(self, feeds_info, dialog, error):
         if feeds_info == None:
             # Something went wrong
             hildon.hildon_banner_show_information(self, '', _('Error finding feeds. Server error.'))
             dialog.destroy()
             return
 
-        news_window = FindWindow(feeds_info, key_words, self)
+        news_window = FindWindow(feeds_info, self)
         news_window.show()
 
         news_window.connect('urls-found', self._find_window_urls_found_cb)
@@ -978,14 +1000,20 @@ class FeedsWindow(hildon.StackableWindow):
         dialog.destroy()
 
     def _sort_ascending_cb(self, button):
+        if not button.get_active():
+            return
         self.settings.feeds_order = constants.ASCENDING_ORDER
         self._sort(constants.ASCENDING_ORDER)
 
     def _sort_descending_cb(self, button):
+        if not button.get_active():
+            return
         self.settings.feeds_order = constants.DESCENDING_ORDER
         self._sort(constants.DESCENDING_ORDER)
 
     def _sort_visits_cb(self, button):
+        if not button.get_active():
+            return
         self.settings.feeds_order = constants.VISITS_ORDER
         self._sort(constants.VISITS_ORDER)
 
@@ -1037,8 +1065,13 @@ class FeedsWindow(hildon.StackableWindow):
 
         chooser.destroy()
 
-    def _import_opml_cb(self, feed_url_list, data):
+    def _import_opml_cb(self, feed_url_list, data, error):
+        if error:
+            hildon.hildon_banner_show_information(self, '', _('Error importing feeds OPML file'))
+            return
+
         if len(feed_url_list) == 0:
+            hildon.hildon_banner_show_information(self, '', _('No feeds to import in OPML file'))
             return
 
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
@@ -1149,13 +1182,16 @@ class EntriesWindow(hildon.StackableWindow):
 
         # Feeds
         self.view = EntriesView(feed_data.feed.title, gtk.HILDON_UI_MODE_NORMAL)
-        self.view.set_model (gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, int))
-        self.view.get_model().set_sort_column_id(self.ENTRIES_MODEL_DATE_PARSED_COLUMN, gtk.SORT_DESCENDING)
+        entries_model = gtk.ListStore(gtk.gdk.Pixbuf, str, gobject.TYPE_PYOBJECT, str, int)
+        entries_model.set_sort_column_id(self.ENTRIES_MODEL_DATE_PARSED_COLUMN, gtk.SORT_DESCENDING)
+        filter_model = entries_model.filter_new()
+        filter_model.set_visible_func(self._row_visible_cb)
+        self.view.set_model (filter_model)
         self.view.connect ("row-activated", self._on_entry_activated)
         self.view.show()
 
         # Draw entries
-        self._draw_entries()
+        self._add_entries()
 
         self.align = gtk.Alignment(0, 0, 1, 1)
         self.align.set_padding(4, 0 , 16, 16)
@@ -1188,8 +1224,18 @@ class EntriesWindow(hildon.StackableWindow):
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
         self._on_connection_changed(self._conn_manager)
 
-    def _draw_entries(self):
-        model = self.view.get_model()
+    def _row_visible_cb(self, model, iter):
+        if self.settings.entries_filter == constants.SHOW_ALL_FILTER:
+            return True
+
+        entry = model.get_value(iter, self.ENTRIES_MODEL_DATA_COLUMN)
+        if not entry:
+            return False
+
+        return not entry.read
+
+    def _add_entries(self):
+        model = self.view.get_model().get_model()
         for entry in self.feed_data.entries:
             entry_iter = model.append()
             model.set(entry_iter,
@@ -1215,6 +1261,23 @@ class EntriesWindow(hildon.StackableWindow):
     def _create_menu(self):
         menu = hildon.AppMenu()
 
+        # Sorting filter
+        self._all_filter_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self._all_filter_button.set_mode(False)
+        self._all_filter_button.set_label(_('Show All'))
+        self._all_filter_button.connect('toggled', self._show_all_cb)
+        menu.add_filter(self._all_filter_button)
+
+        self._unread_filter_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                           group = self._all_filter_button)
+        self._unread_filter_button.set_mode(False)
+        self._unread_filter_button.set_label(_('Show Unread'))
+        self._unread_filter_button.connect('toggled', self._show_unread_cb)
+        menu.add_filter(self._unread_filter_button)
+
+        if self.settings.entries_filter == constants.SHOW_UNREAD_FILTER:
+            self._unread_filter_button.set_active(True)
+
         button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         button.set_label(_('Mark Entries as Read'))
         button.connect('clicked', self._mark_read_cb, True)
@@ -1239,6 +1302,18 @@ class EntriesWindow(hildon.StackableWindow):
         menu.show_all()
         self.set_app_menu(menu)
 
+    def _show_unread_cb(self, button):
+        if not button.get_active():
+            return
+        self.settings.entries_filter = constants.SHOW_UNREAD_FILTER
+        self.view.get_model().refilter()
+
+    def _show_all_cb(self, button):
+        if not button.get_active():
+            return
+        self.settings.entries_filter = constants.SHOW_ALL_FILTER
+        self.view.get_model().refilter()
+
     def _subscribe_feed_cb(self, button):
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
 
@@ -1249,8 +1324,7 @@ class EntriesWindow(hildon.StackableWindow):
         self.manager.subscribe_feed_google(self.feed_data, self.feed_data.href,
                                            not self.feed_data.sync, self._feed_subscribed_cb)
 
-
-    def _feed_subscribed_cb(self, synced, user_data):
+    def _feed_subscribed_cb(self, synced, user_data, error):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
         if not synced:
             # Revert changes
@@ -1277,7 +1351,7 @@ class EntriesWindow(hildon.StackableWindow):
         self._sync_feed_button.set_sensitive(True)
         self._update_feed_button.set_sensitive(True)
 
-    def _sync_google_reader_read_status_cb(self, synced, user_data):
+    def _sync_google_reader_read_status_cb(self, synced, user_data, error):
         if not synced:
             hildon.hildon_banner_show_information(self, '', _('Error syncing read status with Google Reader'))
             return
@@ -1328,7 +1402,7 @@ class EntriesWindow(hildon.StackableWindow):
         # restore normal mode
         self._restore_normal_mode(button)
 
-    def _mark_as_read_sync_cb(self, synced, user_data):
+    def _mark_as_read_sync_cb(self, synced, user_data, error):
         if not synced:
             hildon.hildon_banner_show_information(self, '', _('Error syncing read status with Google Reader'))
 
@@ -1345,7 +1419,7 @@ class EntriesWindow(hildon.StackableWindow):
         item_window = ItemWindow(self.feed_data, self.manager, self.settings, self._conn_manager)
 
         item_window.connect('item-read', self._item_read_cb)
-        item_window.set_item_from_model(treeview.get_model(), path)
+        item_window.set_item_from_model(treeview.get_model().get_model(), path)
         item_window.show()
 
     def _item_read_cb(self, item_window, path):
@@ -1353,7 +1427,7 @@ class EntriesWindow(hildon.StackableWindow):
 
         # Set item as read and update the UI
         entry['read'] = True
-        self.view.get_model()[path][self.ENTRIES_MODEL_TEXT_COLUMN] = \
+        self.view.get_model().get_model()[path][self.ENTRIES_MODEL_TEXT_COLUMN] = \
             self.view.get_visual_entry_text(entry)
 
         # Mark item as read in Google Reader. Do it in an idle as we
@@ -1365,13 +1439,11 @@ class EntriesWindow(hildon.StackableWindow):
         if not self._conn_manager.is_online():
             return
 
-        gtk.gdk.threads_enter()
-        self.manager.mark_as_read_synchronize(self.feed_data.href, entry.link,
-                                              callback=self._mark_as_read_google_cb)
-        gtk.gdk.threads_leave()
+        self.manager.mark_as_read_synchronize(self.feed_data.href, entry.link, True,
+                                              self._mark_as_read_google_cb)
         return False
 
-    def _mark_as_read_google_cb(self, synced, user_data):
+    def _mark_as_read_google_cb(self, synced, user_data, error):
         if not synced:
             hildon.hildon_banner_show_information(self, '', _('Error syncing read status with Google Reader'))
 
@@ -1385,7 +1457,7 @@ class EntriesWindow(hildon.StackableWindow):
         # Ask manager to update feed
         self.manager.update_feed(self.feed_data, self._feed_updated_cb, None)
 
-    def _feed_updated_cb(self, data):
+    def _feed_updated_cb(self, retval, data, error):
         # Update read/unread status if the feed was subscribed
         if self.feed_data.sync:
             hildon.hildon_banner_show_information(self, '', _('Synchronizing read/unread status with Google Reader'))
@@ -1415,7 +1487,7 @@ class EntriesWindow(hildon.StackableWindow):
 
         self.manager.download_all_items(urls, paths_files, self._all_items_downloaded_cb)
 
-    def _all_items_downloaded_cb(self, downloaded):
+    def _all_items_downloaded_cb(self, downloaded, user_data, error):
         if downloaded:
             message = _('Items downloaded')
         else:
@@ -1756,7 +1828,7 @@ class ItemWindow(hildon.StackableWindow):
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
         self.manager.download_item(url_file, folder+file, self._item_downloaded_cb)
 
-    def _item_downloaded_cb(self, downloaded):
+    def _item_downloaded_cb(self, downloaded, user_data, error):
         if downloaded:
             message = _('Item downloaded')
         else:
@@ -1787,7 +1859,7 @@ class FindWindow(hildon.StackableWindow):
     FIND_MODEL_SITE_COLUMN = 0
     FIND_MODEL_URL_COLUMN = 1
 
-    def __init__(self, feedsinfo, key_words, feedsWindow):
+    def __init__(self, feedsinfo, feedsWindow):
         super(FindWindow, self).__init__()
 
         self.feeds_info = feedsinfo
